@@ -13,6 +13,14 @@ use crate::ViewfinderState;
 const BARCODE_TIMEOUT: u32 = 1;
 const PROVIDER_TIMEOUT: u64 = 2;
 
+#[derive(Debug)]
+enum StateChangeState {
+    Equal,
+    Differ,
+    Error,
+    NotDone,
+}
+
 mod imp {
     use std::cell::Cell;
     use std::cell::OnceCell;
@@ -36,6 +44,8 @@ mod imp {
         pub devices: OnceCell<crate::DeviceProvider>,
         pub camera_src: RefCell<Option<gst::Element>>,
         pub camerabin: OnceCell<gst::Element>,
+        pub camera_element: OnceCell<gst::Element>,
+        pub capsfilter: OnceCell<gst::Element>,
         pub sink_paintable: OnceCell<gst::Element>,
         pub tee: OnceCell<crate::PipelineTee>,
         pub bus_watch: OnceCell<gst::bus::BusWatchGuard>,
@@ -134,27 +144,18 @@ mod imp {
                     gst::State::Playing | gst::State::Paused
                 )
             {
-                obj.stop_camerabin();
+                obj.stop_stream();
             }
 
             if let Some(camera) = camera {
-                // `camera_src` might be `None`, which means the element was
-                // reconfigured and we should keep using it
-
-                // TODO This is incredibly not idiomatic.
-                let guard = self.camera_src.borrow();
-                if let Some((bin, camera_src)) = camera.source_element(guard.as_ref()) {
-                    drop(guard);
-                    self.camerabin().set_property("camera-source", &bin);
-                    self.camera_src.replace(Some(camera_src));
+                if let Err(err) = obj.setup_camera_element(&camera) {
+                    log::error!("Could not reconfigure camera element: {err}");
+                    self.set_state(ViewfinderState::Error);
                 }
-
-                let is_front_camera = matches!(camera.location(), crate::CameraLocation::Front);
-                self.is_front_camera.set(is_front_camera);
             }
 
             if obj.is_realized() && matches!(obj.state(), ViewfinderState::Ready) {
-                obj.start_camerabin();
+                obj.start_stream();
             }
 
             obj.notify_camera();
@@ -182,6 +183,7 @@ mod imp {
             let obj = self.obj();
 
             let camerabin = gst::ElementFactory::make("camerabin")
+                .property("location", None::<&str>)
                 .build()
                 .expect("Missing GStreamer Bad Plug-ins");
             self.camerabin.set(camerabin.clone()).unwrap();
@@ -287,6 +289,10 @@ mod imp {
                     log::error!("Could not stop recording: {err}");
                 }
             }
+            if let Err(err) = self.camerabin().set_state(gst::State::Null) {
+                log::error!("Could not stop camerabin: {err}");
+            }
+
             self.picture.unparent();
         }
 
@@ -315,12 +321,12 @@ mod imp {
             self.parent_realize();
 
             if matches!(self.obj().state(), ViewfinderState::Ready) {
-                self.obj().start_camerabin();
+                self.obj().start_stream();
             }
         }
 
         fn unrealize(&self) {
-            self.obj().stop_camerabin();
+            self.obj().stop_stream();
 
             self.parent_unrealize();
         }
@@ -595,17 +601,79 @@ impl Viewfinder {
         );
     }
 
-    fn start_camerabin(&self) {
-        // TODO Present a toast, banner, or message dialog with the error.
-        if let Err(err) = self.imp().camerabin().set_state(gst::State::Playing) {
-            log::error!("Could not start camerabin: {err}");
-            self.imp().set_state(ViewfinderState::Error);
-        } else {
-            log::debug!("Camerabin state succesfully set to PLAYING");
+    /// Starts the viewfinder.
+    pub fn start_stream(&self) {
+        glib::spawn_future_local(glib::clone!(@weak self as obj => async move {
+            obj.change_state_inner(gst::State::Playing).await;
+        }));
+    }
+
+    // It is not needed to call this for gst::State::Null.
+    async fn change_state_inner(&self, state: gst::State) {
+        let (sender, receiver) = futures_channel::oneshot::channel();
+
+        let camerabin = self.imp().camerabin();
+        std::thread::spawn(glib::clone!(@weak camerabin => move || {
+            let timeout = gst::format::ClockTime::from_seconds(2);
+            let (res, current_state, pending_state) = camerabin.state(Some(timeout));
+            let new_state_is = match res {
+                Ok(change_done) => {
+                    if change_done == gst::StateChangeSuccess::Async {
+                        camerabin.set_locked_state(true);
+                        log::debug!("Camerabin could not change its state from {current_state:?} to {pending_state:?}");
+                        StateChangeState::NotDone
+                    } else {
+                        if current_state == state {
+                            StateChangeState::Equal
+                        } else {
+                            StateChangeState::Differ
+                        }
+                    }
+                }
+                Err(err) => {
+                    log::error!("Previous camerabin state changed failed: {err}");
+                    StateChangeState::Error
+                }
+            };
+            sender.send(new_state_is).unwrap();
+        }))
+            .join()
+            .unwrap();
+
+        let change_state = receiver.await.unwrap();
+        match change_state {
+            StateChangeState::Equal => {
+                // Nothing to do, the new state matches the current one.
+            }
+            StateChangeState::NotDone => {
+                log::debug!("Aborting camerabin state change {state:?}");
+                camerabin.abort_state();
+                camerabin.set_locked_state(false);
+                self.set_camerabin_state(state);
+            }
+            // If the previous state change failed, we might as well try to set it now.
+            StateChangeState::Error => self.set_camerabin_state(state),
+            StateChangeState::Differ => self.set_camerabin_state(state),
         }
     }
 
-    fn stop_camerabin(&self) {
+    fn set_camerabin_state(&self, state: gst::State) {
+        match self.imp().camerabin().set_state(state) {
+            Err(err) => {
+                log::error!("Could not start camerabin: {err}");
+                self.imp().set_state(ViewfinderState::Error);
+            }
+            Ok(gst::StateChangeSuccess::Async) => {
+                log::debug!("Trying to set camerabin state to {state:?}")
+            }
+            Ok(_) => log::debug!("Camerabin succesfully state set to {state:?}"),
+        }
+    }
+
+    /// Stops the viewfinder.
+    ///
+    /// A black frame will be shown after this methods has been called.
+    pub fn stop_stream(&self) {
         if let Err(err) = self.imp().camerabin().set_state(gst::State::Null) {
             log::error!("Could not pause camerabin: {err}");
             self.imp().set_state(ViewfinderState::Error);
@@ -781,6 +849,72 @@ impl Viewfinder {
             }),
         );
     }
+
+    fn create_camera_element(
+        &self,
+        device_src: &gst::Element,
+    ) -> Result<gst::Element, glib::BoolError> {
+        use gst::prelude::*;
+
+        let bin = gst::Bin::new();
+
+        let capsfilter = gst::ElementFactory::make("capsfilter").build()?;
+        let decodebin3 = gst::ElementFactory::make("decodebin3").build()?;
+
+        let videoflip = gst::ElementFactory::make("videoflip")
+            .property_from_str("video-direction", "auto")
+            .build()?;
+
+        bin.add_many([&device_src, &capsfilter, &decodebin3, &videoflip])?;
+        gst::Element::link_many([&device_src, &capsfilter, &decodebin3])?;
+
+        self.imp().capsfilter.set(capsfilter).unwrap();
+
+        decodebin3.connect_pad_added(glib::clone!(@weak videoflip => move |_, pad| {
+            if pad.stream().is_some_and(|stream| matches!(stream.stream_type(), gst::StreamType::VIDEO)) {
+                pad.link(&videoflip.static_pad("sink").unwrap())
+                   .expect("Failed to link decodebin3:video_%u pad with videoflip:sink");
+            }
+        }));
+
+        let pad = videoflip.static_pad("src").unwrap();
+        let ghost_pad = gst::GhostPad::with_target(&pad)?;
+        ghost_pad.set_active(true)?;
+
+        bin.add_pad(&ghost_pad)?;
+
+        let wrappercamerabinsrc = gst::ElementFactory::make("wrappercamerabinsrc")
+            .property("video-source", &bin)
+            .build()
+            .expect("Missing GStreamer Bad Plug-ins");
+
+        Ok(wrappercamerabinsrc)
+    }
+
+    fn setup_camera_element(&self, camera: &crate::Camera) -> Result<(), glib::BoolError> {
+        let imp = self.imp();
+
+        if let Some(element) = imp.camera_element.get() {
+            camera.reconfigure(&element)?;
+        } else {
+            let element = camera.create_element()?;
+
+            let wrapper = self.create_camera_element(&element)?;
+            imp.camerabin().set_property("camera-source", &wrapper);
+
+            imp.camera_element.set(element).unwrap();
+        }
+
+        if let Some(capsfilter) = imp.capsfilter.get() {
+            let caps = camera.best_caps();
+            capsfilter.set_property("caps", &caps);
+        }
+
+        let is_front_camera = !matches!(camera.location(), crate::CameraLocation::Back);
+        imp.is_front_camera.set(is_front_camera);
+
+        Ok(())
+    }
 }
 
 fn create_zbar_bin() -> Result<gst::Element, glib::BoolError> {
