@@ -1,20 +1,19 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 #[rustfmt::skip]
 mod config;
+mod application;
 mod enums;
-mod objects;
+mod i18n;
 mod utils;
 mod widgets;
 
+use application::Application;
+use config::{GETTEXT_PACKAGE, LOCALEDIR, RESOURCES_FILE};
 use enums::*;
-use objects::*;
-use widgets::*;
-
 use gettextrs::{gettext, LocaleCategory};
 use gtk::prelude::*;
 use gtk::{gio, glib};
-
-use config::{GETTEXT_PACKAGE, LOCALEDIR, RESOURCES_FILE};
+use widgets::*;
 
 fn main() -> glib::ExitCode {
     // Initialize logger
@@ -25,7 +24,7 @@ fn main() -> glib::ExitCode {
     gettextrs::bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR).expect("Unable to bind the text domain");
     gettextrs::textdomain(GETTEXT_PACKAGE).expect("Unable to switch to the text domain");
 
-    glib::set_application_name(&gettext("Snapshot"));
+    glib::set_application_name(&gettext("Camera"));
 
     let res = gio::Resource::load(RESOURCES_FILE).expect("Could not load gresource file");
     gio::resources_register(&res);
