@@ -2,6 +2,7 @@
 use gtk::prelude::*;
 
 mod camera;
+mod camera_controls;
 mod camera_row;
 mod flash_bin;
 mod gallery;
@@ -9,12 +10,15 @@ mod gallery_button;
 mod gallery_item;
 mod gallery_picture;
 mod gallery_video;
+mod guidelines_bin;
 mod preferences_window;
 mod shutter_button;
+mod sliding_view;
 mod video_player;
 mod window;
 
 pub use camera::Camera;
+pub use camera_controls::CameraControls;
 pub use camera_row::CameraRow;
 pub use flash_bin::FlashBin;
 pub use gallery::Gallery;
@@ -22,8 +26,10 @@ pub use gallery_button::GalleryButton;
 pub use gallery_item::GalleryItem;
 pub use gallery_picture::GalleryPicture;
 pub use gallery_video::GalleryVideo;
+pub use guidelines_bin::GuidelinesBin;
 pub use preferences_window::PreferencesWindow;
 pub use shutter_button::ShutterButton;
+pub use sliding_view::SlidingView;
 pub use video_player::VideoPlayer;
 pub use window::Window;
 
@@ -35,4 +41,5 @@ pub fn init() {
     ShutterButton::static_type();
     GalleryItem::static_type();
     FlashBin::static_type();
+    SlidingView::static_type();
 }
